@@ -31,12 +31,12 @@
 
 #include "buffer.h"
 #include "ffmpegfs.h"
-#include "ffmpeg_utils.h"
 #include "logging.h"
 
 #include <unistd.h>
 #include <sys/mman.h>
 #include <libgen.h>
+#include <cstring>
 
 // Initially Buffer is empty. It will be allocated as needed.
 Buffer::Buffer()
@@ -74,48 +74,60 @@ int Buffer::openio(LPVIRTUALFILE virtualfile)
     return 0;
 }
 
-bool Buffer::open_file(uint32_t segment_no, uint32_t flags)
+bool Buffer::open_file(uint32_t segment_no, uint32_t flags, size_t defaultsize)
 {
+    std::lock_guard<std::recursive_mutex> lock_mutex(m_mutex);
+
     uint32_t index = segment_no;
     if (index)
     {
         index--;
     }
 
-    m_ci[index].m_flags |= flags;
+    CACHEINFO & ci = m_ci[index];
 
-    if (m_ci[index].m_fd != -1)
+    ci.m_flags |= flags;
+
+    if (ci.m_fd != -1)
     {
-        Logging::trace(m_ci[index].m_cachefile, "Cache file is already open.");
+        Logging::trace(ci.m_cachefile, "Cache file is already open.");
+
+        if (defaultsize)
+        {
+            // Make sure the requested size is available
+            reserve(defaultsize);
+        }
         // Already open
         return true;
     }
 
     if (flags & CACHE_FLAG_RW)
     {
-        Logging::debug(m_ci[index].m_cachefile, "Writing to cache file.");
+        Logging::debug(ci.m_cachefile, "Writing to cache file.");
     }
     else
     {
-        Logging::debug(m_ci[index].m_cachefile, "Reading from cache file.");
+        Logging::debug(ci.m_cachefile, "Reading from cache file.");
     }
 
     size_t filesize     = 0;
     bool isdefaultsize  = false;
     uint8_t *p          = nullptr;
 
-    if (!map_file(m_ci[index].m_cachefile, &m_ci[index].m_fd, &p, &filesize, &isdefaultsize, 0, (flags & CACHE_FLAG_RW) ? true : false))
+    if (!map_file(ci.m_cachefile, &ci.m_fd, &p, &filesize, &isdefaultsize, defaultsize, (flags & CACHE_FLAG_RW) ? true : false))
     {
         return false;
     }
 
     if (!isdefaultsize)
     {
-        m_ci[index].m_buffer_pos = m_ci[index].m_buffer_watermark = filesize;
+        ci.m_buffer_pos = ci.m_buffer_watermark = filesize;
     }
 
-    m_ci[index].m_buffer_size       = filesize;
-    m_ci[index].m_buffer            = static_cast<uint8_t*>(p);
+    ci.m_buffer_size        = filesize;
+    ci.m_buffer             = static_cast<uint8_t*>(p);
+    ci.m_buffer_write_size  = 0;
+    ci.m_buffer_writes      = 0;
 
     ++m_cur_open;   // track open files
 
@@ -124,33 +136,37 @@ bool Buffer::open_file(uint32_t segment_no, uint32_t flags)
 
 bool Buffer::close_file(uint32_t segment_no, uint32_t flags)
 {
+    std::lock_guard<std::recursive_mutex> lock_mutex(m_mutex);
+
     uint32_t index = segment_no;
     if (index)
     {
         index--;
     }
 
-    m_ci[index].m_flags &= ~flags;
+    CACHEINFO & ci = m_ci[index];
 
-    if (m_ci[index].m_flags)
+    ci.m_flags &= ~flags;
+
+    if (ci.m_flags)
     {
-        Logging::trace(m_ci[index].m_cachefile, "While attempting to close, the cache file is still in use. Currently open: %1", m_cur_open);
+        Logging::trace(ci.m_cachefile, "While attempting to close, the cache file is still in use. Currently open: %1", m_cur_open);
         return true;
     }
 
-    if (m_ci[index].m_fd == -1)
+    if (ci.m_fd == -1)
     {
         // Already closed
-        Logging::trace(m_ci[index].m_cachefile, "No need to close the unopened cache file. Currently open: %1", m_cur_open);
+        Logging::trace(ci.m_cachefile, "No need to close the unopened cache file. Currently open: %1", m_cur_open);
         return true;
     }
 
-    Logging::trace(m_ci[index].m_cachefile, "Closing cache file.");
+    Logging::trace(ci.m_cachefile, "Closing cache file.");
 
-    bool success = unmap_file(m_ci[index].m_cachefile, &m_ci[index].m_fd, &m_ci[index].m_buffer, m_ci[index].m_buffer_size, &m_ci[index].m_buffer_watermark);
+    bool success = unmap_file(ci.m_cachefile, &ci.m_fd, &ci.m_buffer, ci.m_buffer_size, &ci.m_buffer_watermark);
 
-    m_ci[index].m_buffer_pos    = 0;
-    m_ci[index].m_buffer_size   = 0;
+    ci.m_buffer_pos    = 0;
+    ci.m_buffer_size   = 0;
 
     if (success && m_cur_open > 0)
     {
@@ -162,7 +178,7 @@ bool Buffer::close_file(uint32_t segment_no, uint32_t flags)
 
 bool Buffer::init(bool erase_cache)
 {
-    std::lock_guard<std::recursive_mutex> lck (m_mutex);
+    std::lock_guard<std::recursive_mutex> lock_mutex(m_mutex);
 
     if (is_open())
     {
@@ -228,19 +244,23 @@ bool Buffer::init(bool erase_cache)
 
         delete [] cachefile;
 
-        for (uint32_t index = 0; index < segment_count(); index++)
+#if __cplusplus >= 202002L
+        // C++20 (and later) code
+        for (uint32_t index = 0; CACHEINFO & ci : m_ci)
+#else
+        uint32_t index = 0;
+        for (CACHEINFO & ci : m_ci)
+#endif
         {
-            m_ci[index].m_fd                = -1;
-            m_ci[index].m_buffer            = nullptr;
-            m_ci[index].m_buffer_pos        = 0;
-            m_ci[index].m_buffer_watermark  = 0;
-            m_ci[index].m_buffer_size       = 0;
+            ci.reset();
 
             if (erase_cache)
             {
                 remove_cachefile(index + 1);
                 errno = 0;  // ignore this error
             }
+
+            index++;
         }
 
         // Index only required for frame sets and there is only one.
@@ -258,7 +278,7 @@ bool Buffer::init(bool erase_cache)
             bool isdefaultsize  = false;
             uint8_t *p          = nullptr;
 
-            if (!map_file(m_ci[0].m_cachefile_idx, &m_ci[0].m_fd_idx, &p, &filesize, &isdefaultsize, static_cast<off_t>(sizeof(IMAGE_FRAME)) * virtualfile()->m_video_frame_count, false))
+            if (!map_file(m_ci[0].m_cachefile_idx, &m_ci[0].m_fd_idx, &p, &filesize, &isdefaultsize, sizeof(IMAGE_FRAME) * virtualfile()->m_video_frame_count, false))
             {
                 throw false;
             }
@@ -273,13 +293,9 @@ bool Buffer::init(bool erase_cache)
 
         if (!success)
         {
-            for (uint32_t index = 0; index < segment_count(); index++)
+            for (CACHEINFO & ci : m_ci)
             {
-                m_ci[index].m_fd                  = -1;
-                m_ci[index].m_buffer              = nullptr;
-                m_ci[index].m_buffer_pos          = 0;
-                m_ci[index].m_buffer_watermark    = 0;
-                m_ci[index].m_buffer_size         = 0;
+                ci.reset();
             }
         }
     }
@@ -289,7 +305,7 @@ bool Buffer::init(bool erase_cache)
 
 bool Buffer::set_segment(uint32_t segment_no, size_t size)
 {
-    std::lock_guard<std::recursive_mutex> lck (m_mutex);
+    std::lock_guard<std::recursive_mutex> lock_mutex(m_mutex);
 
     if (!segment_no || segment_no > segment_count())
     {
@@ -302,7 +318,7 @@ bool Buffer::set_segment(uint32_t segment_no, size_t size)
         return false;
     }
 
-    if (!open_file(segment_no, CACHE_FLAG_RW))
+    if (!open_file(segment_no, CACHE_FLAG_RW, size))
     {
         return false;
     }
@@ -315,14 +331,14 @@ bool Buffer::set_segment(uint32_t segment_no, size_t size)
 
 uint32_t Buffer::segment_count()
 {
-    std::lock_guard<std::recursive_mutex> lck (m_mutex);
+    std::lock_guard<std::recursive_mutex> lock_mutex(m_mutex);
 
     return static_cast<uint32_t>(m_ci.size());
 }
 
 uint32_t Buffer::current_segment_no()
 {
-    std::lock_guard<std::recursive_mutex> lck (m_mutex);
+    std::lock_guard<std::recursive_mutex> lock_mutex(m_mutex);
 
     if (!segment_count() || m_cur_ci == nullptr)
     {
@@ -333,7 +349,7 @@ uint32_t Buffer::current_segment_no()
 
 bool Buffer::segment_exists(uint32_t segment_no)
 {
-    std::lock_guard<std::recursive_mutex> lck (m_mutex);
+    std::lock_guard<std::recursive_mutex> lock_mutex(m_mutex);
 
     if (!segment_count() || m_cur_ci == nullptr)
     {
@@ -342,11 +358,18 @@ bool Buffer::segment_exists(uint32_t segment_no)
     return file_exists(m_ci[segment_no - 1].m_cachefile);
 }
 
-bool Buffer::map_file(const std::string & filename, int *fd, uint8_t **p, size_t *filesize, bool *isdefaultsize, off_t defaultsize, bool truncate) const
+bool Buffer::map_file(const std::string & filename, volatile int *fd, uint8_t **p, size_t *filesize, bool *isdefaultsize, size_t defaultsize, bool truncate) const
 {
     bool success = true;
 
-    Logging::trace(filename, "Mapping cache file.");
+    if (!defaultsize)
+    {
+        Logging::trace(filename, "Mapping cache file.");
+    }
+    else
+    {
+        Logging::trace(filename, "Mapping cache file with %1.", format_size(defaultsize).c_str());
+    }
 
     try
     {
@@ -371,22 +394,22 @@ bool Buffer::map_file(const std::string & filename, int *fd, uint8_t **p, size_t
             throw false;
         }
 
-        if (!sb.st_size || *isdefaultsize)
+        if (!sb.st_size || defaultsize)
         {
             // If file is empty or did not exist set file size to default
 
             if (!defaultsize)
             {
-                defaultsize = sysconf(_SC_PAGESIZE);
+                defaultsize = static_cast<size_t>(sysconf(_SC_PAGESIZE));
             }
 
-            if (ftruncate(*fd, defaultsize) == -1)
+            if (ftruncate(*fd, static_cast<off_t>(defaultsize)) == -1)
             {
                 Logging::error(filename, "Error calling ftruncate() to 'stretch' the file: (%1) %2 (fd = %3)", errno, strerror(errno), *fd);
                 throw false;
             }
 
-            *filesize       = static_cast<size_t>(defaultsize);
+            *filesize       = defaultsize;
             *isdefaultsize  = true;
         }
         else
@@ -418,7 +441,7 @@ bool Buffer::map_file(const std::string & filename, int *fd, uint8_t **p, size_t
     return success;
 }
 
-bool Buffer::unmap_file(const std::string &filename, int * fd, uint8_t **p, size_t len, size_t * filesize) const
+bool Buffer::unmap_file(const std::string &filename, volatile int *fd, uint8_t **p, size_t len, size_t * filesize) const
 {
     bool success = true;
 
@@ -470,7 +493,7 @@ bool Buffer::unmap_file(const std::string &filename, int * fd, uint8_t **p, size
 
 bool Buffer::release(int flags /*= CACHE_CLOSE_NOOPT*/)
 {
-    std::lock_guard<std::recursive_mutex> lck (m_mutex);
+    std::lock_guard<std::recursive_mutex> lock_mutex(m_mutex);
 
     bool success = true;
 
@@ -520,12 +543,12 @@ bool Buffer::release(int flags /*= CACHE_CLOSE_NOOPT*/)
 
 bool Buffer::remove_cachefile(uint32_t segment_no) const
 {
-    LPCCACHEINFO ci = !segment_no ? m_cur_ci : &m_ci[segment_no - 1];
-    bool success = remove_file(ci->m_cachefile);
+    const CACHEINFO & ci = !segment_no ? *m_cur_ci : m_ci[segment_no - 1];
+    bool success = remove_file(ci.m_cachefile);
 
-    if (!ci->m_cachefile_idx.empty())
+    if (!ci.m_cachefile_idx.empty())
     {
-        if (!remove_file(ci->m_cachefile_idx))
+        if (!remove_file(ci.m_cachefile_idx))
         {
             success = false;
         }
@@ -535,7 +558,7 @@ bool Buffer::remove_cachefile(uint32_t segment_no) const
 
 bool Buffer::flush()
 {
-    std::lock_guard<std::recursive_mutex> lck (m_mutex);
+    std::lock_guard<std::recursive_mutex> lock_mutex(m_mutex);
 
     if (!segment_count() || m_cur_ci == nullptr || m_cur_ci->m_buffer == nullptr)
     {
@@ -563,38 +586,38 @@ bool Buffer::flush()
 
 bool Buffer::clear()
 {
-    std::lock_guard<std::recursive_mutex> lck (m_mutex);
+    std::lock_guard<std::recursive_mutex> lock_mutex(m_mutex);
 
     bool success = true;
 
-    for (uint32_t n = 0; n < segment_count(); n++)
+    for (CACHEINFO & ci : m_ci)
     {
-        LPCACHEINFO ci = &m_ci[n];
+        ci.m_buffer_pos        = 0;
+        ci.m_buffer_watermark  = 0;
+        ci.m_buffer_size       = 0;
+        ci.m_seg_finished      = false;
+        ci.m_buffer_write_size = 0;
+        ci.m_buffer_writes     = 0;
 
-        ci->m_buffer_pos          = 0;
-        ci->m_buffer_watermark    = 0;
-        ci->m_buffer_size         = 0;
-        ci->m_seg_finished        = false;
-
-        if (ci->m_fd != -1)
+        if (ci.m_fd != -1)
         {
             // If empty set file size to 1 page
             long filesize = sysconf(_SC_PAGESIZE);
 
-            if (ftruncate(ci->m_fd, filesize) == -1)
+            if (ftruncate(ci.m_fd, filesize) == -1)
             {
-                Logging::error(ci->m_cachefile, "Error calling ftruncate() to clear the file: (%1) %2 (fd = %3)", errno, strerror(errno), ci->m_fd);
+                Logging::error(ci.m_cachefile, "Error calling ftruncate() to clear the file: (%1) %2 (fd = %3)", errno, strerror(errno), ci.m_fd);
                 success = false;
             }
         }
         else
         {
-            remove_file(ci->m_cachefile);
+            remove_file(ci.m_cachefile);
         }
 
-        if (ci->m_fd_idx != -1)
+        if (ci.m_fd_idx != -1)
         {
-            memset(ci->m_buffer_idx, 0, ci->m_buffer_size_idx);
+            std::memset(ci.m_buffer_idx, 0, ci.m_buffer_size_idx);
         }
     }
 
@@ -603,7 +626,7 @@ bool Buffer::clear()
 
 bool Buffer::reserve(size_t size)
 {
-    std::lock_guard<std::recursive_mutex> lck (m_mutex);
+    std::lock_guard<std::recursive_mutex> lock_mutex(m_mutex);
 
     if (m_cur_ci == nullptr || m_cur_ci->m_buffer == nullptr)
     {
@@ -640,7 +663,7 @@ bool Buffer::reserve(size_t size)
 
 size_t Buffer::writeio(const uint8_t* data, size_t length)
 {
-    std::lock_guard<std::recursive_mutex> lck (m_mutex);
+    std::lock_guard<std::recursive_mutex> lock_mutex(m_mutex);
 
     if (m_cur_ci == nullptr || m_cur_ci->m_buffer == nullptr)
     {
@@ -655,7 +678,10 @@ size_t Buffer::writeio(const uint8_t* data, size_t length)
     }
     else
     {
-        memcpy(write_ptr, data, length);
+        m_cur_ci->m_buffer_write_size += length;
+        m_cur_ci->m_buffer_writes++;
+
+        std::memcpy(write_ptr, data, length);
         increment_pos(length);
     }
 
@@ -664,7 +690,7 @@ size_t Buffer::writeio(const uint8_t* data, size_t length)
 
 size_t Buffer::write_frame(const uint8_t *data, size_t length, uint32_t frame_no)
 {
-    std::lock_guard<std::recursive_mutex> lck (m_mutex);
+    std::lock_guard<std::recursive_mutex> lock_mutex(m_mutex);
 
     if (data == nullptr || m_cur_ci == nullptr || m_cur_ci->m_buffer_idx == nullptr || frame_no < 1 || frame_no > virtualfile()->m_video_frame_count)
     {
@@ -696,8 +722,8 @@ size_t Buffer::write_frame(const uint8_t *data, size_t length, uint32_t frame_no
     else
     {
         // Create new frame if not existing or not enough space
-        memset(&new_image_frame, 0xFF, sizeof(new_image_frame));
-        memcpy(new_image_frame.m_tag, IMAGE_FRAME_TAG, sizeof(new_image_frame.m_tag));
+        std::memset(&new_image_frame, 0xFF, sizeof(new_image_frame));
+        std::memcpy(new_image_frame.m_tag, IMAGE_FRAME_TAG, sizeof(new_image_frame.m_tag));
         new_image_frame.m_frame_no      = frame_no;
         new_image_frame.m_offset        = buffer_watermark();
         new_image_frame.m_size          = static_cast<uint32_t>(length);
@@ -710,7 +736,7 @@ size_t Buffer::write_frame(const uint8_t *data, size_t length, uint32_t frame_no
             return 0;
         }
 
-        memcpy(reinterpret_cast<void *>(m_cur_ci->m_buffer_idx + start), &new_image_frame, sizeof(IMAGE_FRAME));
+        std::memcpy(reinterpret_cast<void *>(m_cur_ci->m_buffer_idx + start), &new_image_frame, sizeof(IMAGE_FRAME));
     }
 
     return bytes_written;
@@ -856,7 +882,7 @@ bool Buffer::copy(std::vector<uint8_t> * out_data, size_t offset, uint32_t segme
 
 bool Buffer::copy(uint8_t* out_data, size_t offset, size_t bufsize, uint32_t segment_no)
 {
-    std::lock_guard<std::recursive_mutex> lck (m_mutex);
+    std::lock_guard<std::recursive_mutex> lock_mutex(m_mutex);
 
     LPCCACHEINFO ci = const_cacheinfo(segment_no);
 
@@ -872,48 +898,65 @@ bool Buffer::copy(uint8_t* out_data, size_t offset, size_t bufsize, uint32_t seg
         return false;
     }
 
-    size_t segment_size = size(segment_no);
+    size_t segment_size = ci->m_buffer_size;
+
+    if (!segment_size && errno)
+    {
+        Logging::error(ci->m_cachefile, "INTERNAL ERROR: Buffer::copy()! size(segment_no) returned error. Segment: %1 (%2) %3", segment_no, errno, strerror(errno));
+        return false;
+    }
 
     if (ci->m_buffer_size != segment_size)
     {
-        Logging::error(m_cur_ci->m_cachefile, "INTERNAL ERROR: Buffer::copy()! ci->m_buffer_size != segment_size - Segment: %1 ci->m_buffer_size: %2 segment_size: %3", segment_no, ci->m_buffer_size, segment_size);
+        Logging::error(ci->m_cachefile, "INTERNAL ERROR: Buffer::copy()! ci->m_buffer_size != segment_size - Segment: %1 ci->m_buffer_size: %2 segment_size: %3", segment_no, ci->m_buffer_size, segment_size);
         errno = ESPIPE; // Comes from size()
         return false;
     }
 
-    bool success = true;
-
-    if (segment_size >= offset)
+    if (segment_size > offset)
     {
         if (segment_size < offset + bufsize)
         {
             bufsize = segment_size - offset - 1;
         }
 
-        memcpy(out_data, ci->m_buffer + offset, bufsize);
+        std::memcpy(out_data, ci->m_buffer + offset, bufsize);
+
+        return true;
     }
     else
     {
-        Logging::error(m_cur_ci->m_cachefile, "INTERNAL ERROR: Buffer::copy()! size(segment_no) < offset - Segment: %1 Size: %2 Offset: %3", segment_no, segment_size, offset);
+        Logging::error(ci->m_cachefile, "INTERNAL ERROR: Buffer::copy()! segment_size <= offset - Segment: %1 Segment Size: %2 Offset: %3", segment_no, segment_size, offset);
         errno = ESPIPE;
-        success = false;
-    }
 
-    return success;
+        return false;
+    }
 }
 
 bool Buffer::reallocate(size_t newsize)
 {
     if (newsize > size())
     {
-        size_t oldsize = size();
+        if (m_cur_ci->m_buffer_writes)
+        {
+            size_t alloc_size = newsize - size();
+            size_t write_avg = m_cur_ci->m_buffer_write_size / m_cur_ci->m_buffer_writes;
+            size_t write_size = PREALLOC_FACTOR * write_avg;
+            if (write_size > alloc_size)
+            {
+                alloc_size = write_size;
+
+                newsize = size() + alloc_size;
+
+            }
+        }
+
+        Logging::trace(filename(), "Buffer reallocate: %1 -> %2 (Diff %3).", size(), newsize, newsize - size());
 
         if (!reserve(newsize))
         {
             return false;
         }
-
-        Logging::trace(filename(), "Buffer reallocate: %1 -> %2.", oldsize, newsize);
     }
     return true;
 }
@@ -975,7 +1018,7 @@ size_t Buffer::readio(void * /*data*/, size_t /*size*/)
 
 size_t Buffer::read_frame(std::vector<uint8_t> * data, uint32_t frame_no)
 {
-    std::lock_guard<std::recursive_mutex> lck (m_mutex);
+    std::lock_guard<std::recursive_mutex> lock_mutex(m_mutex);
 
     if (data == nullptr || m_cur_ci->m_buffer_idx == nullptr || frame_no < 1 || frame_no > virtualfile()->m_video_frame_count)
     {
@@ -1022,7 +1065,7 @@ void Buffer::closeio()
 
 bool Buffer::have_frame(uint32_t frame_no)
 {
-    std::lock_guard<std::recursive_mutex> lck (m_mutex);
+    std::lock_guard<std::recursive_mutex> lock_mutex(m_mutex);
 
     if (m_cur_ci->m_buffer_idx == nullptr || frame_no < 1 || frame_no > virtualfile()->m_video_frame_count)
     {
@@ -1041,11 +1084,11 @@ bool Buffer::have_frame(uint32_t frame_no)
 
 bool Buffer::is_open()
 {
-    std::lock_guard<std::recursive_mutex> lck (m_mutex);
+    std::lock_guard<std::recursive_mutex> lock_mutex(m_mutex);
 
-    for (uint32_t index = 0; index < segment_count(); index++)
+    for (CACHEINFO & ci : m_ci)
     {
-        if ((m_ci[index].m_fd != -1 && (fcntl(m_ci[index].m_fd, F_GETFL) != -1 || errno != EBADF)))
+        if ((ci.m_fd != -1 && (fcntl(ci.m_fd, F_GETFL) != -1 || errno != EBADF)))
         {
             return true;
         }
