@@ -54,6 +54,14 @@
  */
 class Buffer : public FileIO
 {
+    /**
+     * @brief PREALLOC_FACTOR - Number of elements allocated on reallocate calls
+     * Number of elements allocated on reallocate calls. The buffer keeps track of the
+     * typical buffer write size. To prevent repeated reallocations, reallocate()
+     * attempts to set aside enough memory for PREALLOC FACTOR average elements whenever
+     * it is invoked.
+     */
+    static constexpr int PREALLOC_FACTOR = 5;
 public:
     /**
      * @brief Structure to hold current cache state
@@ -72,12 +80,28 @@ public:
             , m_buffer_idx(nullptr)
             , m_buffer_size_idx(0)
             , m_flags(0)
+            , m_buffer_write_size(0)
+            , m_buffer_writes(0)
         {
         }
 
+        /**
+         * @brief Reset buffer pointers
+         */
+        void reset()
+        {
+            m_fd                = -1;
+            m_buffer            = nullptr;
+            m_buffer_pos        = 0;
+            m_buffer_watermark  = 0;
+            m_buffer_size       = 0;
+            m_buffer_write_size = 0;
+            m_buffer_writes     = 0;
+        }
+
         // Main cache
         std::string             m_cachefile;                    /**< @brief Cache file name */
-        int                     m_fd;                           /**< @brief File handle for buffer */
+        volatile int            m_fd;                           /**< @brief File handle for buffer */
         uint8_t *               m_buffer;                       /**< @brief Pointer to buffer memory */
         size_t                  m_buffer_pos;                   /**< @brief Read/write position */
         size_t                  m_buffer_watermark;             /**< @brief Number of bytes in buffer */
@@ -85,11 +109,14 @@ public:
         bool                    m_seg_finished;                 /**< @brief True if segment completely decoded */
         // Index for frame sets
         std::string             m_cachefile_idx;                /**< @brief Index file name */
-        int                     m_fd_idx;                       /**< @brief File handle for index */
+        volatile int            m_fd_idx;                       /**< @brief File handle for index */
         uint8_t *               m_buffer_idx;                   /**< @brief Pointer to index memory */
         size_t                  m_buffer_size_idx;              /**< @brief Size of index buffer */
         // Flags
         uint32_t                m_flags;                        /**< @brief CACHE_FLAG_* options */
+        // Statistics
+        size_t                  m_buffer_write_size;            /**< @brief Sum of bytes written to the buffer */
+        unsigned int            m_buffer_writes;                /**< @brief Total number of writes to the buffer */
     } CACHEINFO, *LPCACHEINFO;                                  /**< @brief Pointer version of CACHEINFO */
     typedef CACHEINFO const * LPCCACHEINFO;                     /**< @brief Pointer to const version of CACHEINFO */
 
@@ -347,10 +374,11 @@ public:
     /**
      * @brief Open the cache file if not already open.
      * @param[in] segment_no - [0..n-1] Index of segment file number.
-     * @param[in] flags - CACHE_FLAG_* options
+     * @param[in] flags - CACHE_FLAG_* options.
+     * @param[in] defaultsize - If nonzero, after opening, the file will be resized to this value. Works on writeable files only.
      * @return Returns true if the operation was successful or the file was already open; false otherwise.
      */
-    bool                    open_file(uint32_t segment_no, uint32_t flags);
+    bool                    open_file(uint32_t segment_no, uint32_t flags, size_t defaultsize = 0);
     /**
      * @brief If it hasn't already been done, close the cache file.
      * @param[in] segment_no - [0..n-1] Index of segment file number.
@@ -416,7 +444,7 @@ private:
      * @param[out] truncate - If true, when the file is opened, it is truncated.
      * @return Returns true if successful and fd, p, filesize, and isdefaultsize are filled in, or false on error.
      */
-    bool                    map_file(const std::string & filename, int *fd, uint8_t **p, size_t *filesize, bool *isdefaultsize, off_t defaultsize, bool truncate) const;
+    bool                    map_file(const std::string & filename, volatile int *fd, uint8_t **p, size_t *filesize, bool *isdefaultsize, size_t defaultsize, bool truncate) const;
     /**
      * @brief Unmap memory from the file.
      * @param[in] filename - Name of cache file to unmap.
@@ -426,7 +454,7 @@ private:
      * @param[in] filesize - Actual size of the cache file.
      * @return Returns true on success; false on error.
      */
-    bool                    unmap_file(const std::string & filename, int *fd, uint8_t **p, size_t len, size_t *filesize) const;
+    bool                    unmap_file(const std::string & filename, volatile int *fd, uint8_t **p, size_t len, size_t *filesize) const;
 
     /**
      * @brief Get cache information.
