@@ -31,7 +31,6 @@
 
 #include "cache_maintenance.h"
 #include "ffmpegfs.h"
-#include "ffmpeg_utils.h"
 #include "logging.h"
 
 #include <csignal>
@@ -157,7 +156,7 @@ static bool stop_timer()
 {
     Logging::info(nullptr, "Stopping the maintenance timer.");
 
-    if (timer_delete(timerid) == -1)
+    if (timer_delete(timerid) == -1 && errno)
     {
         Logging::error(nullptr, "stop_timer(): timer_delete failed: (%1) %2", errno, strerror(errno));
         return false;
@@ -221,23 +220,13 @@ static bool link_up()
     }
 
     // Also create inter-process semaphore.
-    // First try to open existing semaphore.
-    sem = sem_open(SEM_OPEN_FILE, 0, 0, 0);
+    sem = sem_open(const_cast<const char *>(SEM_OPEN_FILE), O_CREAT, 0777, 1);
 
     if (sem == SEM_FAILED)
     {
-        if (errno == ENOENT)
-        {
-            // If semaphore does not exist, then try to create one.
-            sem = sem_open(const_cast<const char *>(SEM_OPEN_FILE), O_CREAT | O_EXCL, 0777, 1);
-        }
-
-        if (sem == SEM_FAILED)
-        {
-            Logging::error(nullptr, "link_up(): sem_open error (%1) %2", errno, strerror(errno));
-            link_down();
-            return false;
-        }
+        Logging::error(nullptr, "link_up(): sem_open error (%1) %2", errno, strerror(errno));
+        link_down();
+        return false;
     }
 
     return true;
