@@ -60,6 +60,8 @@ extern "C" {
 #include <vector>
 #include <cstring>
 #include <functional>
+#include <chrono>
+#include <thread>
 
 #include <iconv.h>
 #ifdef HAVE_CONFIG_H
@@ -1268,9 +1270,9 @@ int show_caps(int device_only)
     int is_dev;
 
     std::printf("%s\n"
-        " D. = Demuxing supported\n"
-        " .E = Muxing supported\n"
-        " --\n", device_only ? "Devices:" : "File formats:");
+                " D. = Demuxing supported\n"
+                " .E = Muxing supported\n"
+                " --\n", device_only ? "Devices:" : "File formats:");
     last_name = "000";
     for (;;)
     {
@@ -1474,8 +1476,8 @@ FILETYPE get_filetype_from_list(const std::string & desttypelist)
 void init_id3v1(ID3v1 *id3v1)
 {
     // Initialise ID3v1.1 tag structure
-    memset(id3v1, ' ', sizeof(ID3v1));
-    memcpy(&id3v1->m_tag, "TAG", 3);
+    std::memset(id3v1, ' ', sizeof(ID3v1));
+    std::memcpy(&id3v1->m_tag, "TAG", 3);
     id3v1->m_padding = '\0';
     id3v1->m_title_no = 0;
     id3v1->m_genre = 0;
@@ -2529,3 +2531,18 @@ void save_free(void **p)
         free(tmp);
     }
 }
+
+void mssleep(int milliseconds)
+{
+    std::this_thread::sleep_for(std::chrono::milliseconds(milliseconds));
+}
+
+void ussleep(int microseconds)
+{
+    std::this_thread::sleep_for(std::chrono::microseconds(microseconds));
+}
+
+void nssleep(int nanoseconds)
+{
+    std::this_thread::sleep_for(std::chrono::nanoseconds(nanoseconds));
+}
