@@ -56,7 +56,7 @@
  * @li [**Man Pages**](manpages.html)
  * @li [Features In Detail](FEATURES.md)
  * @li [Revision History](HISTORY.md)
- * @li [Installation instructions](INSTALL.md)
+ * @li [Installation Instructions](INSTALL.md)
  * @li [Fixing Problems](PROBLEMS.md)
  * @li @ref todo
  * @li @ref bug
@@ -102,7 +102,11 @@
 #ifdef __cplusplus
 extern "C" {
 #endif
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wmissing-field-initializers"
+#pragma GCC diagnostic ignored "-Wsign-conversion"
 #include <fuse.h>
+#pragma GCC diagnostic pop
 #ifdef __cplusplus
 }
 #endif
@@ -143,7 +147,7 @@ typedef enum HWACCELAPI
     HWACCELAPI_MEDIACODEC,          /**< MediaCodec API   not supported */
 #endif
 #if _WIN32
-// Windows acceleration APIs not supported
+    // Windows acceleration APIs not supported
     HWACCELAPI_DRM,                 /**< DRM              not supported */
     HWACCELAPI_DXVA2,               /**< DXVA2            not supported */
     HWACCELAPI_D3D11VA,             /**< D3D11VA          not supported */
@@ -165,7 +169,6 @@ extern struct FFMPEGFS_PARAMS
     /**
      * @brief Make copy from other FFMPEGFS_PARAMS object.
      * @param[in] other - Reference to source FFmpeg_Frame object.
-     * @return Reference to new FFmpeg_Frame object.
      */
     FFMPEGFS_PARAMS(const FFMPEGFS_PARAMS & other);
     ~FFMPEGFS_PARAMS(); // Do not make virtual, breaks fuse API compatibility!
@@ -279,26 +282,8 @@ extern thread_pool*     tp;
  * @brief Initialise FUSE operation structure.
  */
 void            init_fuse_ops();
-
-/**
- * @brief Custom FFmpeg log function. Used with av_log_set_callback().
- * @param[in] ptr - See av_log_set_callback() in FFmpeg API.
- * @param[in] level - See av_log_set_callback() in FFmpeg API.
- * @param[in] fmt - See av_log_set_callback() in FFmpeg API.
- * @param[in] vl - See av_log_set_callback() in FFmpeg API.
- */
-void            ffmpeg_log(void *ptr, int level, const char *fmt, va_list vl);
 #endif
 
-/**
- * @brief Inititalise logging facility
- * @param[in] logfile - Name of log file if file writing is selected.
- * @param[in] max_level - Maximum level to log.
- * @param[in] to_stderr - If true, log to stderr.
- * @param[in] to_syslog - If true, log to syslog.
- * @return Returns true on success; false on error.
- */
-bool            init_logging(const std::string &logfile, const std::string & max_level, bool to_stderr, bool to_syslog);
 /**
  * @brief Get transcoder cache path.
  * @param[out] path - Path to transcoder cache.
