@@ -246,7 +246,7 @@ static bool is_passthrough(const std::string & ext)
 
 void init_fuse_ops()
 {
-    memset(&ffmpegfs_ops, 0, sizeof(fuse_operations));
+    std::memset(&ffmpegfs_ops, 0, sizeof(fuse_operations));
     ffmpegfs_ops.getattr  = ffmpegfs_getattr;
     ffmpegfs_ops.fgetattr = ffmpegfs_fgetattr;
     ffmpegfs_ops.readlink = ffmpegfs_readlink;
@@ -569,7 +569,7 @@ static int ffmpegfs_getattr(const char *path, struct stat *stbuf)
         const FFmpegfs_Format *current_format = nullptr;
         std::string filename(origpath);
 
-        if (virtual_name(&filename, "", &current_format))
+        if (S_ISREG(stbuf->st_mode) && virtual_name(&filename, "", &current_format))
         {
             if (!current_format->is_multiformat())
             {
@@ -597,7 +597,7 @@ static int ffmpegfs_getattr(const char *path, struct stat *stbuf)
         else
         {
             // Pass-through for regular files
-            Logging::trace(origpath, "getattr: Treating existing file as passthrough.");
+            Logging::trace(origpath, "getattr: Treating existing file/directory as passthrough.");
             errno = 0;
             return 0;
         }
@@ -758,7 +758,7 @@ static int ffmpegfs_getattr(const char *path, struct stat *stbuf)
                                 }
                             }
 
-                            make_hls_fileset(nullptr, nullptr, parent_file->m_destfile + "/", parent_file); // TEST
+                            make_hls_fileset(nullptr, nullptr, parent_file->m_destfile + "/", parent_file);
 
                             LPVIRTUALFILE virtualfile2 = find_original(origpath);
                             if (virtualfile2 == nullptr)
@@ -1093,7 +1093,7 @@ static int ffmpegfs_read(const char *path, char *buf, size_t size, off_t offset,
     size_t locoffset = static_cast<size_t>(offset);  // Cast OK: offset can never be < 0.
     int bytes_read = 0;
 
-    Logging::trace(path, "read: Reading %1 bytes from offset %2.", size, locoffset);
+    Logging::trace(path, "read: Reading %1 bytes from offset %2 to %3.", size, locoffset, size + locoffset);
 
     append_basepath(&origpath, path);
 
@@ -1156,7 +1156,7 @@ static int ffmpegfs_read(const char *path, char *buf, size_t size, off_t offset,
 
         if (bytes)
         {
-            memcpy(buf, &virtualfile->m_file_contents[locoffset], bytes);
+            std::memcpy(buf, &virtualfile->m_file_contents[locoffset], bytes);
         }
 
         bytes_read = static_cast<int>(bytes);
@@ -1240,6 +1240,15 @@ static int ffmpegfs_read(const char *path, char *buf, size_t size, off_t offset,
 
     if (success)
     {
+        if (bytes_read)
+        {
+            Logging::trace(path, "read: Read %1 bytes from offset %2 to %3.", bytes_read, locoffset, static_cast<size_t>(bytes_read) + locoffset);
+        }
+        else
+        {
+            Logging::trace(path, "read: Read output file to EOF.");
+        }
+
         return bytes_read;
     }
     else
@@ -1334,7 +1343,7 @@ static void *ffmpegfs_init(struct fuse_conn_info *conn)
     }
 
     struct sigaction sa;
-    memset(&sa, 0, sizeof(sa));
+    std::memset(&sa, 0, sizeof(sa));
     sigemptyset(&sa.sa_mask);
     sigaddset(&sa.sa_mask, SIGINT);
     sa.sa_handler = sighandler;
@@ -1424,7 +1433,7 @@ static int get_source_properties(const std::string & origpath, LPVIRTUALFILE vir
  */
 static void init_stat(struct stat * stbuf, size_t fsize, time_t ftime, bool directory)
 {
-    memset(stbuf, 0, sizeof(struct stat));
+    std::memset(stbuf, 0, sizeof(struct stat));
 
     stbuf->st_mode = DEFFILEMODE; //S_IFREG | S_IRUSR | S_IRGRP | S_IROTH;
     if (directory)
@@ -1643,7 +1652,7 @@ LPVIRTUALFILE insert_file(VIRTUALTYPE type, const std::string & virtfile, const
         std::string sanitised_origfile(sanitise_filepath(origfile));
         VIRTUALFILE virtualfile;
 
-        memcpy(&virtualfile.m_st, stbuf, sizeof(struct stat));
+        std::memcpy(&virtualfile.m_st, stbuf, sizeof(struct stat));
 
         virtualfile.m_type              = type;
         virtualfile.m_flags             = flags;
@@ -1804,17 +1813,17 @@ int load_path(const std::string & path, const struct stat *statbuf, void *buf, f
 
                 stat_set_size(&virtualfile->m_st, static_cast<size_t>(stbuf2.st_size));
 
-                memcpy(&stbuf, &virtualfile->m_st, sizeof(struct stat));
+                std::memcpy(&stbuf, &virtualfile->m_st, sizeof(struct stat));
             }
             else
             {
                 if (statbuf == nullptr)
                 {
-                    memcpy(&stbuf, &virtualfile->m_st, sizeof(struct stat));
+                    std::memcpy(&stbuf, &virtualfile->m_st, sizeof(struct stat));
                 }
                 else
                 {
-                    memcpy(&stbuf, statbuf, sizeof(struct stat));
+                    std::memcpy(&stbuf, statbuf, sizeof(struct stat));
 
                     stat_set_size(&stbuf, static_cast<size_t>(virtualfile->m_st.st_size));
                 }
@@ -2017,15 +2026,15 @@ static int make_hls_fileset(void * buf, fuse_fill_dir_t filler, const std::strin
         //"index_3_a.m3u8\n";
 
         master_contents = "#EXTM3U\n"
-                "#EXT-X-STREAM-INF:PROGRAM-ID=1\n"
-                "index_0_av.m3u8\n";
+                          "#EXT-X-STREAM-INF:PROGRAM-ID=1\n"
+                          "index_0_av.m3u8\n";
 
         strsprintf(&index_0_av_contents, "#EXTM3U\n"
-                "#EXT-X-TARGETDURATION:%i\n"
-                "#EXT-X-ALLOW-CACHE:YES\n"
-                "#EXT-X-PLAYLIST-TYPE:VOD\n"
-                "#EXT-X-VERSION:3\n"
-                "#EXT-X-MEDIA-SEQUENCE:1\n", static_cast<int32_t>(params.m_segment_duration / AV_TIME_BASE));
+                                         "#EXT-X-TARGETDURATION:%i\n"
+                                         "#EXT-X-ALLOW-CACHE:YES\n"
+                                         "#EXT-X-PLAYLIST-TYPE:VOD\n"
+                                         "#EXT-X-VERSION:3\n"
+                                         "#EXT-X-MEDIA-SEQUENCE:1\n", static_cast<int32_t>(params.m_segment_duration / AV_TIME_BASE));
 
         int64_t remaining_duration  = virtualfile->m_duration % params.m_segment_duration;
         size_t  segment_size        = virtualfile->m_predicted_size / virtualfile->get_segment_count();
@@ -2306,7 +2315,7 @@ static int parse_file(LPVIRTUALFILE newvirtualfile)
         struct stat stbuf;
         if (lstat(newvirtualfile->m_origfile.c_str(), &stbuf) == 0)
         {
-            memcpy(&newvirtualfile->m_st, &stbuf, sizeof(stbuf));
+            std::memcpy(&newvirtualfile->m_st, &stbuf, sizeof(stbuf));
         }
 
         for (unsigned int stream_idx = 0; stream_idx < format_ctx->nb_streams; stream_idx++)
